@@ -1,7 +1,9 @@
 from .core import Application
+from .core import Middleware
 from .router import Router
 from .request import HttpRequest
 from .response import HttpResponse
+from .response import HttpResponseFile
 from .sapi import json_api
 from .sapi import jsonp_api
 from .sapi import simplejson_api
