@@ -1,21 +1,31 @@
-
+import typing
 from functools import cached_property
 from wsgiref.util import FileWrapper
 
 from miniweb.exceptions import PayloadTooLarge
+from miniweb.exceptions import LengthRequired
 
 from .request import HttpRequest
 from .response import HttpResponse
+from .response import HttpChunkResponseData
 from .router import Router
 
+
+class Middleware(object):
+    def __init__(self, get_response:typing.Callable[[HttpRequest, HttpResponse], None]) -> None:
+        self.get_response = get_response
+    
+    def __call__(self, http_request:HttpRequest, http_response:HttpResponse) -> None:
+        return self.get_response(http_request, http_response)
+
 class Application(object):
 
-    def __init__(self,):
+    def __init__(self):
         self.middlewares = []
         self.router = Router()
         self.global_permitted_methods = []
 
-    def set_middlewares(self, middlewares):
+    def set_middlewares(self, middlewares:typing.List[Middleware]):
         self.middlewares = [] + middlewares
 
     @cached_property
@@ -25,7 +35,7 @@ class Application(object):
             dispatch = middleware_class(dispatch)
         return dispatch
 
-    def dispatch(self, http_request, http_response):
+    def dispatch(self, http_request:HttpRequest, http_response:HttpResponse) -> None:
         if self.global_permitted_methods and (not http_request.method in self.global_permitted_methods):
             http_response.not_allowed(method=http_request.method, permitted_methods=self.global_permitted_methods)
         else:
@@ -35,20 +45,24 @@ class Application(object):
             else:
                 try:
                     handler(http_request, http_response)
+                except LengthRequired as error:
+                    http_response.length_required()
                 except PayloadTooLarge as error:
                     http_response.payload_too_large(error.args[0], error.args[1])
 
     def __call__(self, env, start_response):
-        request = HttpRequest(env, application=self)
-        response = HttpResponse(start_response, request=request, application=self)
-        self.dispatch_chain(request, response)
-        return self.do_final_response(request, response)
+        http_request = HttpRequest(env, application=self)
+        http_response = HttpResponse(start_response, request=http_request, application=self)
+        self.dispatch_chain(http_request, http_response)
+        return self.do_final_response(http_request, http_response)
 
-    def do_final_response(self, request, response):
-        response.start_response(response.final_status_code, response.final_headers)
-        final_content = response.final_content
+    def do_final_response(self, http_request:HttpRequest, http_response:HttpResponse):
+        http_response.start_response(http_response.final_status_code, http_response.final_headers)
+        final_content = http_response.final_content
         if not final_content:
             return []
         if isinstance(final_content, FileWrapper):
             return final_content
-        return [final_content]
\ No newline at end of file
+        if isinstance(final_content, HttpChunkResponseData):
+            return final_content
+        return [final_content]
