@@ -14,6 +14,8 @@ from .exceptions import PayloadTooLarge
 
 class HttpRequest(object):
 
+    CONTENT_LENGTH_HEADER_REQUIRED_METHODS = ["POST", "PUT"] # POST and PUT request requires Content-Length header.
+
     post_size_limit = 1024*1024*2
     temporary_path = None
 
@@ -158,7 +160,7 @@ class HttpRequest(object):
         self.is_content_ready = True
         self.is_files_ready = True
 
-    def read_input(self, read_all_files=False):
+    def read_input(self):
         if self.is_content_ready and self.is_files_ready:
             return
 
@@ -168,12 +170,12 @@ class HttpRequest(object):
             self.set_post_data_empty()
             return
 
-        if self.content_length is None:
+        if (self.method in self.CONTENT_LENGTH_HEADER_REQUIRED_METHODS) and (self.content_length is None):
             # deal with content length missing
             self.set_post_data_empty()
             raise LengthRequired()
 
-        if self.content_length > self.post_size_limit:
+        if (self.method in self.CONTENT_LENGTH_HEADER_REQUIRED_METHODS) and (self.content_length > self.post_size_limit):
             # deal with payload too large
             self.set_post_data_empty()
             raise PayloadTooLarge(self.content_length, self.post_size_limit)
